package com.googlecode.marrowboy.writers;

import java.io.PrintStream;
import java.util.Date;

import org.apache.commons.lang.StringEscapeUtils;

import com.googlecode.marrowboy.assertionresult.AssertionResult;
import com.googlecode.marrowboy.assertionresult.ClassResult;
import com.googlecode.marrowboy.assertionresult.MethodResult;
import com.googlecode.marrowboy.assertionresult.TestRunResult;

/**
 * {@link ReportWriter} implementation which writes out the report in HTML
 * format.
 * 
 * @author mjg
 */
public class HtmlWriter implements ReportWriter {

	// TODO: This class is an outrage! Can't we use a Velocity template or
	// something?

	private final CssProvider cssProvider;

	/**
	 * Constructor, which sets a new {@link DefaultCssProvider}
	 */
	public HtmlWriter() {
		this.cssProvider = new DefaultCssProvider();
	}

	/**
	 * {@inheritDoc} <br/>
	 * This implementation makes badly formatted HTML output.
	 */
	public void writeReport(TestRunResult testRunResult, PrintStream stream) {

		stream.println("<html><head><style type=\"text/css\">");
		stream.println(cssProvider.getCss());
		stream.println("</style></head><body>");

		stream.println("<h1>MarrowBoy Report</h1>");

		stream.println("<div class='box resultsBox'>");

		stream.println("<span class='boilerplate'>Run at</span> <span class='detail'>" + new Date() + "</span><br/>");

		stream.println("<span class='boilerplate'>In total</span> ");
		stream.println("<span class='detail'>" + testRunResult.getPassCount() + "/" + testRunResult.getTotalCount() + "</span> <span class='boilerplate'>assertions passed</span><br/>");

		stream.println("</div>");

		stream.println("<h1>Tests run</h1>");

		stream.println("<div class='box testListBox'>");

		int linkCnt = 0;
		stream.println("<ul>");
		for (ClassResult thisClassResult : testRunResult.getClassResults().values()) {

			String className = thisClassResult.getClassName();

			stream.println("<li>");
			stream.println("<a href='#" + linkCnt + className + "'><span class='classLink'>" + className + "</span></a>");

			String classPassResult = thisClassResult.getFailCount() == 0 ? "passed" : "failed";
			stream.println("<span class='" + classPassResult + "'>" + thisClassResult.getPassCount() + "/" + thisClassResult.getTotalCount() + "</span>");

			linkCnt++;

			stream.println("<ul>");
			for (MethodResult methodResult : thisClassResult.getMethodResults()) {

				String methodName = methodResult.getMethodName();

				stream.println("<li>");

				String methodPassResult = methodResult.getFailCount() == 0 ? "passed" : "failed";
				stream.println("<span class='" + methodPassResult + "'>" + methodResult.getPassCount() + "/" + methodResult.getTotalCount() + "</span>");

				stream.println("<a href='#" + linkCnt + methodName + "'><span class='methodLink'>" + methodName + "</span></a>");

				stream.println("</li>");

				linkCnt++;
			}
			stream.println("</ul>");

			stream.println("</li>");
		}
		stream.println("</ul>");

		stream.println("</div>");

		stream.println("<h1>Test Results</h1>");

		linkCnt = 0;
		for (ClassResult thisClassResult : testRunResult.getClassResults().values()) {
			String className = thisClassResult.getClassName();

			stream.println("<h2><a name='" + linkCnt + className + "'>" + StringEscapeUtils.escapeHtml(className) + "</a></h2>");
			linkCnt++;

			for (MethodResult methodResult : thisClassResult.getMethodResults()) {

				String methodName = methodResult.getMethodName();

				stream.println("<h3><a name='" + linkCnt + methodName + "'>" + StringEscapeUtils.escapeHtml(methodName) + "</a></h3>");
				linkCnt++;

				for (AssertionResult thisAssertion : methodResult.getAssertionResults()) {

					String passOrFail = (thisAssertion.wasSuccessful() ? "passed" : "failed");

					stream.println("<div class='assertionResult " + passOrFail + "'>");
					stream.println("<table>");

					String reason = thisAssertion.getReason();
					if (reason != null && !reason.isEmpty()) {
						stream.println("<tr><td align='right'>");
						stream.println("<span class='boilerplate'>Because</span></td><td><span class='detail'>" + StringEscapeUtils.escapeHtml(reason) + "</span><br/>");
						stream.println("</tr></td>");
					}

					stream.println("<tr><td align='right'>");
					stream.println("<span class='boilerplate'>We expected</span></td><td><span class='detail'>" + StringEscapeUtils.escapeHtml(thisAssertion.getMatcher()) + "</span><br/>");
					stream.println("</tr></td>");

					String gotResult = StringEscapeUtils.escapeHtml(thisAssertion.getActual());
					if (gotResult.contains("\n")) {
						gotResult = "<pre class='detail'>" + gotResult + "</pre>";
					}

					stream.println("<tr><td align='right'>");
					stream.println("<span class='boilerplate'>And got</span></td><td><span class='detail'>" + gotResult + "</span><br/>");
					stream.println("</tr></td>");

					stream.println("<tr><td align='right'>");
					stream.println("<span class='boilerplate'>So this test</span></td><td><span class='detail'>" + passOrFail);

					if (!thisAssertion.wasSuccessful()) {
						stream.println("in " + thisAssertion.getFilename() + " on line " + thisAssertion.getLineNumber());
					}

					stream.println("</span></tr></td>");

					stream.println("</table>");
					stream.println("</div>");

				}

			}

		}

		stream.println("</body></html>");
	}
}
