package com.googlecode.marrowboy.writers;

import java.io.PrintStream;
import java.util.Date;

import com.googlecode.marrowboy.assertionresult.AssertionResult;
import com.googlecode.marrowboy.assertionresult.ClassResult;
import com.googlecode.marrowboy.assertionresult.MethodResult;
import com.googlecode.marrowboy.assertionresult.TestRunResult;

/**
 * {@link ReportWriter} implementation which writes out the report in plain text
 * format.
 * 
 * @author mjg
 */
public class PlainTextWriter implements ReportWriter {

	private static final String INDENT = "    "; // 4 spaces :)
	private static final String NEWLINE = System.getProperty("line.separator");

	/**
	 * Constructor.
	 */
	public PlainTextWriter() {
	}

	/**
	 * {@inheritDoc} <br/>
	 * This implementation makes formatted simple text output, using
	 * <code>System.getProperty("line.separator")</code> to split lines.
	 */
	public void writeReport(TestRunResult testRunResult, PrintStream stream) {

		stream.println("MarrowBoy report generated at " + new Date());
		stream.println();
		stream.println(INDENT + testRunResult.getPassCount() + "/" + testRunResult.getTotalCount() + " assertions passed");

		for (ClassResult thisClassResult : testRunResult.getClassResults().values()) {
			String className = thisClassResult.getClassName();

			stream.println();
			stream.println();
			stream.println(className);

			for (MethodResult thisMethodResult : thisClassResult.getMethodResults()) {

				stream.println();
				stream.println();
				stream.println(INDENT + thisMethodResult.getMethodName());
				stream.println();

				for (AssertionResult thisAssertion : thisMethodResult.getAssertionResults()) {

					String message = formatAssertion(thisAssertion);

					stream.println(message);

				}
			}
		}
	}

	private String formatAssertion(AssertionResult thisAssertion) {

		StringBuilder builder = new StringBuilder();

		if (!thisAssertion.getReason().isEmpty()) {
			builder.append(INDENT).append(INDENT);
			builder.append(thisAssertion.getReason());
			builder.append(":").append(NEWLINE);
		}

		builder.append(INDENT).append(INDENT);
		builder.append("Expected: " + thisAssertion.getMatcher() + NEWLINE);
		builder.append(INDENT).append(INDENT);
		builder.append("Got:      " + thisAssertion.getActual());
		builder.append((thisAssertion.wasSuccessful() ? " (PASS) " : " (FAIL! " + thisAssertion.getFilename() + ":" + thisAssertion.getLineNumber() + ") "));
		builder.append(NEWLINE).append(NEWLINE);

		return builder.toString();

	}
}
